//************************************************************************
//*         DemoSql v 1.1  -  demonstrator for BipFile functions         *
//*                                                                      *
//*                                                                      *
//* Licence : NFM                                                        *
//************************************************************************

#pragma comment (lib, "comctl32.lib")


#include <windows.h>
#include <string>


// Include for Bip
#include "BipGDI.h"
#include "BipFILE.h"
#include "BipDATE.h"
#include "BipNUM.h"


	using namespace std;



//-------------------------------------------------------------
//!        Global variables                                   !
//-------------------------------------------------------------
	const int MAX_RECORD		= 100;		// max record per query
	const int MAX_FIELD			= 50;		// max field(column) per query
	const string HEADER_FORM	= "fheader";
	const string MAIN_WINDOW	= "main";
	const string RECORD_WINDOW	= "record";
	const string DBNAME_ACCESS	= "Microsoft Jet - Access (*.mdb)";
	const string DBNAME_SQLSRV	= "Microsoft SQL Server";
	const string DBNAME_MYSQL	= "MySQL Server";
	const string QUERY_NAME		= "bipquery";

	string sRecord [MAX_RECORD];
	string sField [MAX_FIELD];		// field name
	int iFLength [MAX_FIELD];		// field length
	string sFType [MAX_FIELD];		// field type



//-------------------------------------------------------------
//!        Read records from table                            !
//-------------------------------------------------------------
int Read_Table (void)
{
	int iError;
	string sError;
	int iRecord;
	int iField;
	long lLong;
	double dDouble;
	BP_Date dtDate;
	__int64 i64Freq;
	__int64 i64Begin;
	__int64 i64End;
	BP_Currency cCurrency;
	LONGLONG llLongLong;
	string sDbName;
	string sDbType;
	string sServer;
	string sPort;
	string sUser;
	string sPassword;
	string sSqlQuery;
	bool bTrust;
	string sParam;
	string sString;
	BF_STRUCT_FIELD TmpField;


	// get database type
	sDbType = BG_FormItemValueGetString (HEADER_FORM, "dbtype");

	// get database name
	sDbName = BG_FormItemValueGetString (HEADER_FORM, "dbname");

	// get server name
	sServer = BG_FormItemValueGetString (HEADER_FORM, "server");

	// get port number
	sPort = BG_FormItemValueGetString (HEADER_FORM, "port");

	// get sql query
	sSqlQuery = BG_FormItemValueGetString (HEADER_FORM, "sqlquery");

	// get trusted connection
	bTrust = BG_FormItemValueGetBool (HEADER_FORM, "trust");

	// get user name
	sUser = BG_FormItemValueGetString (HEADER_FORM, "user");

	// get password
	sPassword = BG_FormItemValueGetString (HEADER_FORM, "psw");


	// disable error
	BF_ErrorDisplaySet (false, false);

	// close table - in case of previous error
	iError = BF_TableClose (QUERY_NAME);


	// display text in status bar
	iError = BG_StatusbarTextSet (HEADER_FORM, "Running ....");

	// select database type
	if (sDbType == DBNAME_ACCESS)
		sString = BF_DB_ACCESS;
	else if (sDbType == DBNAME_SQLSRV)
		sString = BF_DB_SQLSERVER;
	else if (sDbType == DBNAME_MYSQL)
		sString = BF_DB_MYSQL;

	// fill parameter string
	if (bTrust)
		sParam = BF_PRM_TRUST + "=true";
	else
		sParam = BF_PRM_USER + "=" + sUser + ";" + BF_PRM_PSW + "=" + sPassword;

	sParam += ";" + BF_PRM_PORT + "=" + sPort;

	// initialize chrono
	QueryPerformanceFrequency ( (LARGE_INTEGER*) &i64Freq);
	QueryPerformanceCounter ( (LARGE_INTEGER*) &i64Begin);

	// launch query
	sError = BF_SQLQueryOpen (QUERY_NAME, sSqlQuery, sDbName, sServer, sString, sParam);
	if (sError == "")
	{
		// get chrono
		QueryPerformanceCounter ( (LARGE_INTEGER*) &i64End);

		// display duration in status bar
		sString = "Query duration (ms) : "
			+ BF_IntToString (((i64End - i64Begin) * 1000) / i64Freq);
		iError = BG_StatusbarTextSet (HEADER_FORM, sString);


		iError = 0;

		// clear field name and length
		for (iField = 0; iField < MAX_FIELD; iField++)
		{
			sField [iField] = "";
			iFLength [iField] = 0;
		}

		// get fields name and length
		iField = 0;
		while (iField < MAX_FIELD && iError == 0)
		{
			TmpField = BF_SQLQueryFieldGet (QUERY_NAME, iField + 1);
			if (TmpField.sDbFieldRemark != "")
			{
				// save field name
				sField [iField] = TmpField.sDbFieldRemark;

				// save field length
				iFLength [iField] = TmpField.iDbFieldLength;
				if (TmpField.sDbFieldRemark.length () > iFLength [iField])
					iFLength [iField] = TmpField.sDbFieldRemark.length ();
				// length is 19 for date
				if (TmpField.sInterFieldType == BF_FLD_DATETIME)
					iFLength [iField] = 19;

				// save field type
				sFType [iField] = TmpField.sInterFieldType;
			}
			else
				break;

			iField++;
		}

		// clear saved record
		for (iRecord = 0; iRecord < MAX_RECORD; iRecord++)
			sRecord [iRecord] = "";

		// read records
		iRecord = 0;
		while (iRecord < MAX_RECORD && iError == 0)
		{
			// read one record
			iError = BF_RecordReadNext (QUERY_NAME);

			// read all fields of the record
			iField = 0;
			while (sField [iField] != "" && iField < MAX_FIELD && iError == 0)
			{
				if (sFType [iField] == BF_FLD_STRING)
					sString = BF_FieldValueGetString (QUERY_NAME, sField [iField]);
				else if (sFType [iField] == BF_FLD_BYTE ||
						 sFType [iField] == BF_FLD_INTEGER ||
						 sFType [iField] == BF_FLD_LONG)
				{
					lLong = BF_FieldValueGetInteger (QUERY_NAME, sField [iField]);
					sString = BF_IntToString (lLong);
				}
				else if (sFType [iField] == BF_FLD_LONGLONG)
				{
					llLongLong = BF_FieldValueGetLongLong (QUERY_NAME, sField [iField]);
					sString = BF_LongLongToString (llLongLong);
				}
				else if (sFType [iField] == BF_FLD_FLOAT)
				{
					dDouble = BF_FieldValueGetFloat (QUERY_NAME, sField [iField]);
					sString = BF_DoubleToString (dDouble);
				}
				else if (sFType [iField] == BF_FLD_DATETIME)
				{
					dtDate = BF_FieldValueGetDate (QUERY_NAME, sField [iField]);
					dtDate.setformat ("yyyy/mm/dd hh:nn:ss");
					sString = dtDate.str ();
				}
				else if (sFType [iField] == BF_FLD_CURRENCY)
				{
					cCurrency = BF_FieldValueGetCurrency (QUERY_NAME, sField [iField]);
					cCurrency.setstdformat (15, 4, "US");
					sString = cCurrency.str ();
				}
				else
					sString = "";

				// align string on column size
				sString.resize (iFLength [iField], ' ');
				sString += ' ';

				sRecord [iRecord] += sString;
				iField++;
			}

			iRecord++;
		}

		// close table
		iError = BF_TableClose (QUERY_NAME);
	}

	// display error
	if (sError != "")
	{
		// display texte in status bar
		iError = BG_StatusbarTextSet (HEADER_FORM, "Error :");

		iError = BF_StringToInt (sError);
		sField [0] = "";
		dtDate.setnow ();
		sRecord [0] = dtDate.str ("dd/mm/yyyy hh:nn:ss");
		if (iError < 1000)
			sRecord [1] = "Error in SQL Query - bad Table ?";
		else if (iError < 1200)
			sRecord [1] = "Error during connection to the DataBase";
		else if (iError < 1300)
			sRecord [1] = "Error during connection - bad Database, Server, Port or User/Password ?";
		else
			sRecord [1] = "Error in SQL Query";
		iField = sError.find_last_of ("]") + 1;
		if (iField > 1)
		{
			sRecord [2] = "  error code : " + sError.substr (0, iField);
			sRecord [3] = "               " + sError.substr (iField);
			sRecord [4] = "";
		}
		else
		{
			sRecord [2] = "  error code : " + sError;
			sRecord [3] = "";
		}
	}


	return 0;
}


//-------------------------------------------------------------
//!        Launch SQL command                                 !
//-------------------------------------------------------------
int Launch_Command (void)
{
	int iError;
	int iField;
	string sError;
	BP_Date dtDate;
	string sDbName;
	string sDbType;
	string sServer;
	string sPort;
	string sUser;
	string sPassword;
	string sSqlCommand;
	bool bTrust;
	string sParam;
	string sString;
	BF_STRUCT_FIELD TmpField;


	// get database type
	sDbType = BG_FormItemValueGetString (HEADER_FORM, "dbtype");

	// get database name
	sDbName = BG_FormItemValueGetString (HEADER_FORM, "dbname");

	// get server name
	sServer = BG_FormItemValueGetString (HEADER_FORM, "server");

	// get port number
	sPort = BG_FormItemValueGetString (HEADER_FORM, "port");

	// get sql command
	sSqlCommand = BG_FormItemValueGetString (HEADER_FORM, "sqlquery");

	// get trusted connection
	bTrust = BG_FormItemValueGetBool (HEADER_FORM, "trust");

	// get user name
	sUser = BG_FormItemValueGetString (HEADER_FORM, "user");

	// get password
	sPassword = BG_FormItemValueGetString (HEADER_FORM, "psw");


	// disable error
	BF_ErrorDisplaySet (false, false);

	// close table - in case of previous error
	iError = BF_TableClose (QUERY_NAME);


	// select database type
	if (sDbType == DBNAME_ACCESS)
		sString = BF_DB_ACCESS;
	else if (sDbType == DBNAME_SQLSRV)
		sString = BF_DB_SQLSERVER;
	else if (sDbType == DBNAME_MYSQL)
		sString = BF_DB_MYSQL;

	// fill parameter string
	if (bTrust)
		sParam = BF_PRM_TRUST + "=true";
	else
		sParam = BF_PRM_USER + "=" + sUser + ";" + BF_PRM_PSW + "=" + sPassword;

	sParam += ";" + BF_PRM_PORT + "=" + sPort;

	// launch command
	sError = BF_SQLCommand (sSqlCommand, sDbName, sServer, sString, sParam);

	sField [0] = "";
	dtDate.setnow ();
	sRecord [0] = "SQL Command : " + dtDate.str ("dd/mm/yyyy hh:nn:ss");
	sRecord [1] = "Database says :";
	if (sError == "")
	{
		sRecord [2] = "  OK";
		sRecord [3] = "";
	}
	else
	{
		iField = sError.find_last_of ("]") + 1;
		if (iField > 1)
		{
			sRecord [2] = "  error code : " + sError.substr (0, iField);
			sRecord [3] = "               " + sError.substr (iField);
			sRecord [4] = "";
		}
		else
		{
			sRecord [2] = "  error code : " + sError;
			sRecord [3] = "";
		}
	}


	return 0;
}



//-------------------------------------------------------------
//!        Procedure for header form                          !
//-------------------------------------------------------------
int Header_Proc (const string sBGFormName, const string sBGItemName, const string sBGMessage,
				HWND hBGCurWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	int iError;
	string sString;
	BG_STRUCT_WINDOW TmpWindow;


	// one button is clic
	if (sBGMessage == BG_MSG_ITEMCLIC)
	{
		// it is "go" button
		if (sBGItemName == "go")
		{
			// get sql query
			sString = BG_FormItemValueGetString (HEADER_FORM, "sqlquery");
			sString = sString.substr (0,6);

			if (BF_StringToUpper (sString) == "SELECT")
			{
				// query : read records 
				iError = Read_Table ();
			}
			else
			{
				// execute command
				iError = Launch_Command ();
			}


			// force redraw of record window
			TmpWindow = BG_WindowGet (RECORD_WINDOW);
			iError = InvalidateRect (TmpWindow.hBGWnd, NULL, true);

			return 0;
		}
	}
	// item has change
	if (sBGMessage == BG_MSG_ITEMCHANGE)
	{
		// database type
		if (sBGItemName == "dbtype")
		{
			// enable or disable server item
			if (BG_FormItemValueGetString (HEADER_FORM, "dbtype") != DBNAME_ACCESS)
				iError = BG_FormItemModify (HEADER_FORM, "server", BG_PRM_ENABLE , "true");
			else
				iError = BG_FormItemModify (HEADER_FORM, "server", BG_PRM_ENABLE , "false");

			// enable or disable port number item
			if (BG_FormItemValueGetString (HEADER_FORM, "dbtype") == DBNAME_MYSQL)
				iError = BG_FormItemModify (HEADER_FORM, "port", BG_PRM_ENABLE , "true");
			else
				iError = BG_FormItemModify (HEADER_FORM, "port", BG_PRM_ENABLE , "false");

			return 0;
		}
		// trusted connection
		else if (sBGItemName == "trust")
		{
			// enable or disable server item
			if (BG_FormItemValueGetBool (HEADER_FORM, "trust") == false)
			{
				iError = BG_FormItemModify (HEADER_FORM, "user", BG_PRM_ENABLE , "true");
				iError = BG_FormItemModify (HEADER_FORM, "psw", BG_PRM_ENABLE , "true");
			}
			else
			{
				iError = BG_FormItemModify (HEADER_FORM, "user", BG_PRM_ENABLE , "false");
				iError = BG_FormItemModify (HEADER_FORM, "psw", BG_PRM_ENABLE , "false");
			}

			return 0;
		}
	}
	// form was just created
	else if (sBGMessage == BG_MSG_FORMOPENAFT)
	{
		// initialize combo values
		iError = BG_FormItemModify (HEADER_FORM, "dbtype", BG_PRM_ADDVALUE, DBNAME_ACCESS);
		iError = BG_FormItemModify (HEADER_FORM, "dbtype", BG_PRM_ADDVALUE, DBNAME_SQLSRV);
		iError = BG_FormItemModify (HEADER_FORM, "dbtype", BG_PRM_ADDVALUE, DBNAME_MYSQL);
		return 0;
	}


	return 0;
}



//-------------------------------------------------------------
//!        Display form                                       !
//-------------------------------------------------------------
void Display_Form (void)
{
	int iError;
	int iItem;


	iItem = 1;

	// header
	iError = BG_FormItemAdd (HEADER_FORM, "dbtypelib", BG_ITEM_STATIC, iItem++, 5, 5, 200, 20,
							"value=DataBase Type :;ftname=Arial;ftsize=10;txcol=blue;ftbold=true");
	iError = BG_FormItemAdd (HEADER_FORM, "dbtype", BG_ITEM_COMBOSYS, iItem++, 5, 25, 220, 100,
							"remark=DataBase type");

	iError = BG_FormItemAdd (HEADER_FORM, "dbnamelib", BG_ITEM_STATIC, iItem++, 250, 5, 200, 20,
							"value=DataBase Name :;ftname=Arial;ftsize=10;txcol=blue;ftbold=true");
	iError = BG_FormItemAdd (HEADER_FORM, "dbname", BG_ITEM_EDIT, iItem++, 250, 25, 150, 24,
							"value=Northwind;remark=DataBase Name");

	iError = BG_FormItemAdd (HEADER_FORM, "serverlib", BG_ITEM_STATIC, iItem++, 425, 5, 200, 20,
							"value=Server Name :;ftname=Arial;ftsize=10;txcol=blue;ftbold=true");
	iError = BG_FormItemAdd (HEADER_FORM, "server", BG_ITEM_EDIT, iItem++, 425, 25, 150, 24,
							"enable=false;remark=Server Name");

	iError = BG_FormItemAdd (HEADER_FORM, "portlib", BG_ITEM_STATIC, iItem++, 600, 5, 200, 20,
							"value=Port :;ftname=Arial;ftsize=10;txcol=blue;ftbold=true");
	iError = BG_FormItemAdd (HEADER_FORM, "port", BG_ITEM_EDIT, iItem++, 600, 25, 50, 24,
							"value=3306;remark=Port number;enable=false");

	iError = BG_FormItemAdd (HEADER_FORM, "trustlib", BG_ITEM_STATIC, iItem++, 5, 55, 200, 20,
							"value=Trusted connection :;ftname=Arial;ftsize=10;txcol=blue;ftbold=true;txvpos=center");
	iError = BG_FormItemAdd (HEADER_FORM, "trust", BG_ITEM_CHECK, iItem++, 140, 55, 20, 20,
							"check=true;bkcol=white;txcol=blue;brdsize=2;remark=Trusted connection");

	iError = BG_FormItemAdd (HEADER_FORM, "userlib", BG_ITEM_STATIC, iItem++, 180, 55, 200, 20,
							"value=User name :;ftname=Arial;ftsize=10;txcol=blue;ftbold=true;txvpos=center");
	iError = BG_FormItemAdd (HEADER_FORM, "user", BG_ITEM_EDIT, iItem++, 260, 55, 100, 24,
							"value=sa;enable=false;remark=User name");

	iError = BG_FormItemAdd (HEADER_FORM, "pswlib", BG_ITEM_STATIC, iItem++, 380, 55, 200, 20,
							"value=Password :;ftname=Arial;ftsize=10;txcol=blue;ftbold=true;txvpos=center");
	iError = BG_FormItemAdd (HEADER_FORM, "psw", BG_ITEM_EDIT, iItem++, 455, 55, 100, 24,
							"enable=false;remark=Password;password=true");


	// SQL Query
	iError = BG_FormItemAdd (HEADER_FORM, "sqllib", BG_ITEM_STATIC, iItem++, 5, 85, 600, 20,
							"value=SQL Query (select ...) or SQL Command (create,update,delete,...) :;ftname=Arial;ftsize=12;txcol=yellow;ftbold=true;txvpos=center");
	iError = BG_FormItemAdd (HEADER_FORM, "sqlquery", BG_ITEM_EDIT, iItem++, 5, 105, 780, 70,
							"value=select * from ;multiline=true;bkcol=A0FFFF;remark=Enter here your SQL Query");

	// GO button
	iError = BG_FormItemAdd (HEADER_FORM, "go", BG_ITEM_BUTTON, iItem++, 700, 20, 80, 60,
							"value=GO;bkcol=red;txcol=white;ftname=Arial;ftsize=16;ftbold=true;remark=Launch the query");

	// display header form
	iError = BG_FormCreate (HEADER_FORM, "bkcol=E7B096;edge=false",
							0, 0, 0, 200, Header_Proc);

}



//-------------------------------------------------------------
//!        Procedure for record form                          !
//-------------------------------------------------------------
LRESULT CALLBACK Record_Proc (HWND hBGCurWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	int iError;
	int iRecord;
	int iField;
	string sString;


	if (message == WM_PAINT)
	{
		// select window
		iError = BG_WindowSelect (RECORD_WINDOW);

		// change font
		iError = BG_TextFontSet ("Courier New", 8);

		// display field name
		iField = 0;
		while (sField [iField] != "" && iField < MAX_FIELD)
		{
			sString = sField [iField];
			sString.resize (iFLength [iField], ' ');
			iError = BG_TextDraw (sString + ' ');

			iField++;
		}

		if (sField [0] != "")
		{
			// line feed
			iError = BG_TextDraw ("", "lf=true");

			// display separator line
			sString.erase ();
			sString.resize (255, '-');
			iError = BG_TextDraw (sString, "lf=true");
		}

		// display records
		iRecord = 0;
		while (sRecord [iRecord] != "" && iRecord < MAX_RECORD)
		{
			iError = BG_TextDraw (sRecord [iRecord], "lf=true");
			iRecord++;
		}
			
		// release current window
		iError = BG_WindowRelease ();

		return 0;
	}


	// automatic message
	iError = BG_WindowMessageAuto (message, wParam);

	return DefWindowProc (hBGCurWnd, message, wParam, lParam);
}



//-------------------------------------------------------------
//!    Procedure for main window                              !
//-------------------------------------------------------------
LRESULT CALLBACK WndProc (HWND hBGCurWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	int iError;


	// automatic message treatment
	iError = BG_WindowMessageAuto (message, wParam);

	return DefWindowProc (hBGCurWnd, message, wParam, lParam);

}



//-------------------------------------------------------------
//!        BEGINING OF PROGRAM                                !
//-------------------------------------------------------------
int APIENTRY WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine,
					  int nCmdShow)
{
	int iError;


	// create main window
	iError = BG_WindowMainCreate (MAIN_WINDOW, nCmdShow, WndProc,
					"name=DemoSQL v1.1 - read only " + BG_IntToString (MAX_RECORD) +
					" first records and " + BG_IntToString (MAX_FIELD) + " first fields." +
					" - Licence NFM" + ";resize=false",
					0, 0, 800, 600);
	if (iError != 0)
		return iError;


	// create record window
	iError = BG_WindowChildCreate (RECORD_WINDOW, Record_Proc, "bkcol=F3DACD",
					0, 200, 0, 375);
	
	
	// create modeless form
	Display_Form ();


	// enter message loop
	return BG_MessageLoop();

}
